@@ -17,7 +17,7 @@ package Enbugger;
 # these licenses.
 
 BEGIN {
-    $VERSION = '2.014';
+    $VERSION = '2.015';
 }
 
 use XSLoader ();
@@ -82,7 +82,7 @@ use Carp ();
 use Scalar::Util ();
 
 # Public class settings.
-use vars qw( $DefaultDebugger );
+use vars qw( $DefaultDebugger %DBsub );
 
 use constant (); # just to load it.
 
@@ -354,9 +354,9 @@ sub initialize_dbline {
      }
 
      if ( not defined $file ) {
-         # WTF?
          *DB::dbline = [];
-         *DB::dbline = {}; # TODO magic(L)
+         *DB::dbline = {};
+         Enbugger::set_magic_dbfile( \%DB::dbline );
      }
      else {
          no strict 'refs';
@@ -392,7 +392,8 @@ sub load_file {
         local $/ = "\n";
         *$glob = [
             map { Scalar::Util::dualvar( 0, $_ ) }
-            ( undef, readline $fh )
+            ( "BEGIN { require 'perl5db.pl' } # Generated by " . __FILE__,
+	      readline $fh )
         ];
     }
 
@@ -416,12 +417,40 @@ sub load_file {
 sub instrument_runtime {
     # Now do the *real* work.
     my ( $class ) = @_;
-    
+
+    # PL_DBgv = gv_fetchpvs("DB::DB", GV_ADDMULTI, SVt_PVGV);
+    eval 'sub DB::DB {}' if ! defined &DB::DB;
+
     # Load the source code for all loaded files. Too bad about (eval 1)
     # though. This doesn't work. Why not!?!
     $class->load_source;
-    
+
+    # PL_DBsingle = GvSV((gv_fetchpvs("DB::single", GV_ADDMULTI, SVt_PV)));
+    # if (!SvIOK(PL_DBsingle))
+    #     sv_setiv(PL_DBsingle, 0);
+    $DB::single = 0 if ! defined $DB::single;
+
+    # PL_DBtrace = GvSV((gv_fetchpvs("DB::trace", GV_ADDMULTI, SVt_PV)));
+    # if (!SvIOK(PL_DBtrace))
+    #     sv_setiv(PL_DBtrace, 0);
+    $DB::trace = 0 if ! defined $DB::trace;
+
+    # PL_DBsignal = GvSV((gv_fetchpvs("DB::signal", GV_ADDMULTI, SVt_PV)));
+    # if (!SvIOK(PL_DBsignal))
+    #     sv_setiv(PL_DBsignal, 0);
+    $DB::signal = 0 if ! defined $DB::signal;
+
+    # Walk over all optrees.
+    # * Transform nextstate COP* nodes to dbstate COP* nodes as appropriate
+    # * Set ${"main::_<$file"}[X] array elements with COP* pointers
+    # * Capture function name start/end line numbers
     B::Utils::walkallops_simple( \ &Enbugger::instrument_op );
+
+    # Provide %DB::sub.
+    %DB::sub =
+        map { $_ => sprintf '%s:%d-%d', @{$DBsub{$_}} }
+        keys %DBsub;
+    undef %DBsub;
 }
 
 
@@ -437,19 +466,22 @@ sub instrument_op {
         # @{"_<$file"} entries where there are COP entries are
         # dualvars of pointers to the COP nodes that will get
         # OPf_SPECIAL toggled to indicate breakpoints.
-        {
-            my $file = $op->file;
-            my $line = $op->line;
-            my $ptr  = $$op;
+	my $ptr  = $$op;
+	my $source = do {
+	    no strict 'refs';
+	    \ @{"main::_<$B::Utils::file"};
+	};
+	if ( $ptr ) {
+	    $source->[$B::Utils::line] = Scalar::Util::dualvar( $ptr, $source->[$B::Utils::line] );
+	}
 
-            my $source = do {
-                no strict 'refs';
-                \ @{"main::_<$file"};
-            };
-            if ( $ptr ) {
-                $source->[$line] = Scalar::Util::dualvar( $ptr, $source->[$line] );
-            }
-        }
+	if ($DBsub{$B::Utils::sub}) {
+	    $DBsub{$B::Utils::sub}[1] = $B::Utils::line if $B::Utils::line < $DBsub{$B::Utils::sub}[1];
+	    $DBsub{$B::Utils::sub}[2] = $B::Utils::line if $B::Utils::line > $DBsub{$B::Utils::sub}[2];
+	}
+	else {
+	    $DBsub{$B::Utils::sub} = [ $B::Utils::file, ($B::Utils::line) x 2 ];
+	}
 
         #print $op->file ."\t".$op->line."\t".$o->stash->NAME."\t";
         # Disable or enable debugging for this opcode.
